@@ -17,7 +17,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <popt.h>
 #include "GConfX.h"
 #include "gconf.h"
 #include "gconf-internals.h"
@@ -45,6 +44,45 @@ static void gconf_detach_config_server(void);
 /* Maximum number of times to try re-spawning the server if it's down. */
 #define MAX_RETRIES 1
 
+/* copied from gutf8.c where it exists as a (unfortunately) non-exported function */
+static gchar *
+utf8_make_valid (const gchar *name)
+{
+  GString *string;
+  const gchar *remainder, *invalid;
+  gint remaining_bytes, valid_bytes;
+  
+  string = NULL;
+  remainder = name;
+  remaining_bytes = strlen (name);
+  
+  while (remaining_bytes != 0) 
+    {
+      if (g_utf8_validate (remainder, remaining_bytes, &invalid)) 
+        break;
+      valid_bytes = invalid - remainder;
+    
+      if (string == NULL) 
+        string = g_string_sized_new (remaining_bytes);
+
+      g_string_append_len (string, remainder, valid_bytes);
+      /* append U+FFFD REPLACEMENT CHARACTER */
+      g_string_append (string, "\357\277\275");
+      
+      remaining_bytes -= valid_bytes + 1;
+      remainder = invalid + 1;
+    }
+  
+  if (string == NULL)
+    return g_strdup (name);
+  
+  g_string_append (string, remainder);
+
+  g_assert (g_utf8_validate (string->str, -1, NULL));
+  
+  return g_string_free (string, FALSE);
+}
+
 gboolean
 gconf_key_check(const gchar* key, GError** err)
 {
@@ -54,15 +92,17 @@ gconf_key_check(const gchar* key, GError** err)
     {
       if (err)
         *err = gconf_error_new (GCONF_ERROR_BAD_KEY,
-                                _("Key \"%s\" is NULL"),
-                                key);
+				_("Key is NULL"));
       return FALSE;
     }
   else if (!gconf_valid_key (key, &why))
     {
-      if (err)
+      if (err) {
+        gchar *utf8_key = utf8_make_valid (key);
         *err = gconf_error_new (GCONF_ERROR_BAD_KEY, _("\"%s\": %s"),
-                                key, why);
+                                utf8_key, why);
+        g_free (utf8_key);
+      }
       g_free(why);
       return FALSE;
     }
@@ -1897,21 +1937,8 @@ gconf_clear_cache(GConfEngine* conf, GError** err)
   
   if (gconf_engine_is_local(conf))
     {
-      GError* error = NULL;
-      
       gconf_sources_clear_cache(conf->local_sources);
       
-      if (error != NULL)
-        {
-          if (err)
-            *err = error;
-          else
-            {
-              g_error_free(error);
-            }
-          return;
-        }
-      
       return;
     }
 
@@ -2295,7 +2322,9 @@ gconf_detach_config_server(void)
  * @void: 
  * 
  * Detach from the config server and release
- * all related resources
+ * all related resources.
+ *
+ * Returns: 1 if an exception occurs, 0 otherwise.
  **/
 int
 gconf_debug_shutdown (void)
@@ -2542,9 +2571,16 @@ gconf_postinit (gpointer app, gpointer mod_info)
 /* All deprecated */
 const char gconf_version[] = VERSION;
 
-struct poptOption gconf_options[] = {
-  {NULL}
-};
+struct 
+{
+    const char * longName;
+    char shortName;
+    int argInfo;
+    void * arg;
+    int val;
+    const char * descrip;
+    const char * argDescrip;
+} gconf_options[] = { { NULL } };
 
 /* Also deprecated */
 gboolean     
@@ -2621,14 +2657,15 @@ gconf_valid_key      (const gchar* key, gchar** why_invalid)
       else
         {
           const gchar* inv = invalid_chars;
+          guchar c = (unsigned char) *s;
 
           just_saw_slash = FALSE;
           
-          if (((unsigned char)*s) > 127)
+          if (c > 127)
             {
               if (why_invalid != NULL)
-                *why_invalid = g_strdup_printf (_("'%c' is not an ASCII character, so isn't allowed in key names"),
-                                                *s);
+                *why_invalid = g_strdup_printf (_("'\\%o' is not an ASCII character, so isn't allowed in key names"),
+                                                (guint) c);
               return FALSE;
             }
           
@@ -2747,11 +2784,11 @@ gconf_unescape_key (const char *escaped_key,
           if (*p == '@')
             {
               /* *p is the @ that ends a seq */
-              char *end;
+              char *end_seq;
               guchar val;
               
-              val = strtoul (start_seq, &end, 10);
-              if (start_seq != end)
+              val = strtoul (start_seq, &end_seq, 10);
+              if (start_seq != end_seq)
                 g_string_append_c (retval, val);
               
               start_seq = NULL;
@@ -3497,47 +3534,33 @@ corba_errno_to_gconf_errno(ConfigErrorType corba_err)
     {
     case ConfigFailed:
       return GCONF_ERROR_FAILED;
-      break;
     case ConfigNoPermission:
       return GCONF_ERROR_NO_PERMISSION;
-      break;
     case ConfigBadAddress:
       return GCONF_ERROR_BAD_ADDRESS;
-      break;
     case ConfigBadKey:
       return GCONF_ERROR_BAD_KEY;
-      break;
     case ConfigParseError:
       return GCONF_ERROR_PARSE_ERROR;
-      break;
     case ConfigCorrupt:
       return GCONF_ERROR_CORRUPT;
-      break;
     case ConfigTypeMismatch:
       return GCONF_ERROR_TYPE_MISMATCH;
-      break;
     case ConfigIsDir:
       return GCONF_ERROR_IS_DIR;
-      break;
     case ConfigIsKey:
       return GCONF_ERROR_IS_KEY;
-      break;
     case ConfigOverridden:
       return GCONF_ERROR_OVERRIDDEN;
-      break;
     case ConfigLockFailed:
       return GCONF_ERROR_LOCK_FAILED;
-      break;
     case ConfigNoWritableDatabase:
       return GCONF_ERROR_NO_WRITABLE_DATABASE;
-      break;
     case ConfigInShutdown:
       return GCONF_ERROR_IN_SHUTDOWN;
-      break;
     default:
       g_assert_not_reached();
       return GCONF_ERROR_SUCCESS; /* warnings */
-      break;
     }
 }
 
@@ -3548,7 +3571,6 @@ gconf_server_broken(CORBA_Environment* ev)
     {
     case CORBA_SYSTEM_EXCEPTION:
       return TRUE;
-      break;
 
     case CORBA_USER_EXCEPTION:
       {
@@ -3558,11 +3580,9 @@ gconf_server_broken(CORBA_Environment* ev)
 
         return ce->err_no == ConfigInShutdown;
       }
-      break;
       
     default:
       return FALSE;
-      break;
     }
 }
 
@@ -3574,14 +3594,12 @@ gconf_handle_corba_exception(CORBA_Environment* ev, GError** err)
     case CORBA_NO_EXCEPTION:
       CORBA_exception_free (ev);
       return FALSE;
-      break;
     case CORBA_SYSTEM_EXCEPTION:
       if (err)
         *err = gconf_error_new (GCONF_ERROR_NO_SERVER, _("CORBA error: %s"),
                                 CORBA_exception_id (ev));
       CORBA_exception_free (ev);
       return TRUE;
-      break;
     case CORBA_USER_EXCEPTION:
       {        
         ConfigException* ce;
@@ -3594,11 +3612,9 @@ gconf_handle_corba_exception(CORBA_Environment* ev, GError** err)
         CORBA_exception_free (ev);
         return TRUE;
       }
-      break;
     default:
       g_assert_not_reached();
       return TRUE;
-      break;
     }
 }
 
